<?php

require_once dirname(__FILE__) . '/../../lib/phoneprovisioning.php';
require_once 'Gigaset.php';

class GigasetPhoneProvisioning extends VirtualPhoneProvisioningController
{

    private $sipAccount = null;
    private $firstProvisioning = false;

    const MAX_DEVICES_N720 = 100;

    protected function getProvURL($mac = null)
    {
        if (!$mac) {
            $mac = $this->mMacAddress;
        }

        $url = parent::getProvURL();
        $url = $url . "$mac.xml";
        return $url;
    }

    protected function getPhonebookURL()
    {
        $url = parent::getProvURL();
        return $url . "pb/$this->mMacAddress.xml?";
    }

    protected function checkMacAddress($mac)
    {
        return parent::checkMacAddress($mac);
    }

    protected function checkUserAgent($ua)
    {
        //$ua = "N720-DM-PRO/70.108.00.000.000;7C2F80XXXXXX";
        //new Gigaset ua => Gigaset N720 DM PRO/70.110.00.000.000;7C2F80XXXXXX
        //N510 IP PRO/42.231.00.000.000;7C2F80XXXXXX
        //Gigaset N870 IP PRO/83.V2.14.0+build.9387b8e;589EC6XXXXXX

        $knownPhoneModel = Gigaset::getSupportedPhoneModels();
        Log::d($ua . " contacs us", "gigaset-dls");

        $firstExplode = explode('/', $ua);
        $phoneModelName = str_replace(' ', '-', $firstExplode[0]);
        $phoneModelName = str_replace('Gigaset-', '', $phoneModelName);

        if (!array_key_exists($phoneModelName, $knownPhoneModel)) {
            return false;
        }

        $secondExplode = explode(';', $firstExplode[1]);
        $firmwareString = $secondExplode[0];

        $vParts = explode(".", $firmwareString);

        $this->setCurrentPhoneModel($knownPhoneModel[$phoneModelName]);
        $firmwareVersion = $this->getFirmwareVersion($phoneModelName, $vParts);
        $this->setCurrentFirmwareVersion($firmwareVersion);
        return true;
    }

    protected function getFirmwareVersion($phoneModelName, $vParts)
    {
        $vPartsNormal = array();
        if (startsWith($phoneModelName, 'N870')) {
            if (isset($vParts[3])) {
                $vParts[3] = substr($vParts[3], 0, strpos($vParts[3], "+"));
            }

            $vPartsNormal[] = str_pad($vParts[0], 2, '0', STR_PAD_LEFT);
            $vPartsNormal[] = str_pad($vParts[1], 2, '0', STR_PAD_LEFT);
            $vPartsNormal[] = str_pad($vParts[2], 2, '0', STR_PAD_LEFT);
            $vPartsNormal[] = str_pad($vParts[3], 1, '0', STR_PAD_LEFT);
        } else {
            $vPartsNormal[] = str_pad($vParts[0], 2, '0', STR_PAD_LEFT);
            $vPartsNormal[] = str_pad($vParts[1], 3, '0', STR_PAD_LEFT);
            $vPartsNormal[] = str_pad($vParts[2], 2, '0', STR_PAD_LEFT);
            $vPartsNormal[] = str_pad($vParts[3], 3, '0', STR_PAD_LEFT);
            $vPartsNormal[] = str_pad($vParts[4], 3, '0', STR_PAD_LEFT);
        }
        return implode(".", $vPartsNormal);
    }

    protected function loadConfigValues($mac = NULL)
    {
        $this->getTemplate(); //loads the template into the current Settings Array

        $Phone = $this->getPhoneModel();
        $phoneType = $Phone->type;
        $this->sipAccount = $Phone->getSipAccount();
        if (empty($Phone->last_prov_dateTime)) {
            $this->firstProvisioning = true;
        }

        if (startsWith($phoneType, 'N510')) {
            $this->loadConfigValuesForN510();
        } else if (startsWith($phoneType, 'N720')) {
            $this->loadConfigValuesForN720();
        } else if (startsWith($phoneType, 'N870')) {
            $this->loadConfigValuesForN870();
        } else {
            return false;
        }
        if (!startsWith($phoneType, 'N870')) {
            $this->hexSetting('BS_AE_SwConfig.ucCountryCodeTone', $this->getToneScheme());
            $this->setting('BS_VOIP_Data.ul_SIP_AllowMethodMask', '0x373e');
            $this->setting('BS_CUSTOM_ORG.bit.bEct', '1'); //Enable call transfer by On-Hook;
            $this->stringSetting('BS_VOIP_Data.aucS_SIPAlertInfoHeaderInternal', 'alert-internal');
            $this->setting('BS_IP_Data.ucB_ADD_SIPID_TO_HTTP_REQ', '1');
        }

        $provisioningProfile = ProvisioningProfile::model()->findByPk($Phone->provisioning_profile);

        if ($provisioningProfile) {
            $params = $provisioningProfile->getParamsForPhone($phoneType);
            $paramKeys = array_keys($this->mPhonalisaProvisioningVariables);
            foreach ($params as $param) {
                $param->value = str_replace($paramKeys, $this->mPhonalisaProvisioningVariables, $param->value);

                $this->setting($param->name, $param->value);
                Log::d("Overwriting key from parameter profile: " . $param->name . " = " . $param->value, "gigaset-dls");
            }
        }
        if (startsWith($phoneType, 'N870')) {
            if ($this->firstProvisioning) {
                $this->n870Setting("update_dm", 'local', '<oper>');
                $this->n870Setting("RegStart", time(), 'param');
                $this->n870Setting("RegDuration", 86400, 'param</oper>');
            }
        }
        return true;
    }

    private function loadConfigValuesForN510()
    {
        $SipAccount = $this->sipAccount;

        //NTP-Settings
        $this->setting('BS_IP_Data1.ucB_TIME_USE_AUTOMATIC_NTP_SYN', '1');
        $this->stringSetting('BS_IP_Data1.aucS_TIME_NTP_SERVER[0]', cfg('NTP_HOST'));
        $this->stringSetting('BS_IP_Data1.aucS_TIME_NTP_SERVER[40]', cfg('NTP_HOST'));
        $this->setting('BS_IP_Data1.ucB_TIME_USE_AUTOMATIC_DST', '1');
        $this->hexSetting('BS_IP_Data1.uiI_TIME_TIMEZONE', $this->getTimeZone());

        // XML-Phonebook
        $this->stringSetting('BS_XML_Netdirs.astNetdirProvider[1].aucServerURL', $this->getPhonebookURL());
        $this->stringSetting('BS_XML_Netdirs.astNetdirProvider[1].aucWhitePagesDirName', __('ADDRESSESS_FORM_HEADER_BUSINESS'));
        $this->stringSetting('BS_XML_Netdirs.astNetdirProvider[1].aucPrivatePagesDirName', __('ADDRESSESS_FORM_HEADER_PRIVATE'));
        $this->setting('BS_XML_Netdirs.aucActivatedNetdirs[0]', $this->gigasetHexFormat(20) . ',' . $this->gigasetHexFormat(21));
        $this->setting('BS_XML_Netdirs.aucAvailableNetdirs[0]', $this->gigasetHexFormat(20) . ',' . $this->gigasetHexFormat(21));

        $this->setting('BS_IP_Data1.ucB_USE_R_KEY_FOR_CALL_TRANSFER', '1');
        $this->hexSetting('BS_IP_Data1.ucI_DIALING_PLAN_COUNTRY_ID', $this->getDialingPlanCountryID());
        $this->setting('BS_IP_Data1.ucI_HTTPLANGUAGE', $this->getLanguage());

        $registrationExpireTime = $SipAccount->expires;
        $this->configureSipAccountN510($SipAccount, 1, $registrationExpireTime);

        $Phone = $this->getPhoneModel();
        $this->removeOvermuchPhones($Phone);
        // enumerating devices in database
        for ($i = 2; $i < $this->getMaxPhones($Phone) + 1; $i++) {
            $phone = Phones::model()->findByAttributes(array("mac_addr" => $this->mMacAddress . "-" . ($i - 1)));

            if (!$phone) {
                $phone = $this->getAdditionalPhone($i - 1);
            }

            if (!$phone) {
                $this->setting('BS_IP_Data1.ucB_SIP_ACCOUNT_IS_ACTIVE_' . $i, '0');
                $this->setting('BS_IP_Data1.ucB_VOIP_NET_AM_ENABLED_' . $i, '0');
                $this->hexSetting('BS_IP_Data1.uiI_RE_REGISTRATION_TIMER_' . $i, $registrationExpireTime);
            } else {
                $SipAccount = $phone->getSipAccount();
                $this->configureSipAccountN510($SipAccount, $i, $registrationExpireTime);
            }
        }
    }

    private function configureSipAccountN510($SipAccount, $index, $registrationExpireTime)
    {
        $this->setting('BS_IP_Data1.ucB_SIP_ACCOUNT_IS_ACTIVE_' . $index, '1');
        $this->setting('BS_IP_Data1.ucB_VOIP_NET_AM_ENABLED_' . $index, '1');
        $this->stringSetting('BS_IP_Data1.aucS_SIP_ACCOUNT_NAME_' . $index, $SipAccount->authname);
        $this->stringSetting('BS_IP_Data1.aucS_VOIP_NET_AM_NUMBER_' . $index, cfg('DP_VOICEMAIL_LISTENING_PREFIX'));

        //Audio-Codec-Settings
        $this->setting('BS_IP_Data1.ucI_SIP_PREFERRED_VOCODER_' . $index, $this->getPhoneCodecs());
        $this->setting('BS_IP_Data1.ucI_SIP_AVAILABLE_VOCODER_' . $index, $this->getPhoneCodecs());

        $keyIndex = '_' . $index;
        if ((int) $index == 1) {
            $keyIndex = '';
        }
        $index = (int) $index - 1;
        $this->stringSetting('BS_Accounts.astAccounts[' . $index . '].aucAccountName[0]', $SipAccount->authname);
        $this->stringSetting('BS_AE_Subscriber.stMtDat[' . $index . '].aucTlnName[0]', $SipAccount->authname);
        $this->stringSetting('BS_IP_Data1.aucS_SIP_DISPLAYNAME' . $keyIndex, $SipAccount->authname);
        $this->stringSetting('BS_IP_Data3.aucS_SIP_LOGIN_ID' . $keyIndex, $SipAccount->authname);
        $this->stringSetting('BS_IP_Data1.aucS_SIP_PASSWORD' . $keyIndex, $SipAccount->secret);
        $this->stringSetting('BS_IP_Data1.aucS_SIP_USER_ID' . $keyIndex, $SipAccount->authname);
        $this->hexSetting('BS_IP_Data1.uiI_RE_REGISTRATION_TIMER' . $keyIndex, $registrationExpireTime);

        //Provisioning-Settings
        $this->stringSetting('BS_IP_Data1.aucS_SIP_PROVIDER_NAME' . $keyIndex, cfg('PHONALISA_NAME'));
        $this->stringSetting('BS_IP_Data1.aucS_SIP_DOMAIN' . $keyIndex, $SipAccount->getSIPRegistrar());
        $this->stringSetting('BS_IP_Data1.aucS_SIP_SERVER' . $keyIndex, $SipAccount->getSIPRegistrar());
        $this->hexSetting('BS_IP_Data1.uiI_SIP_SERVER_PORT' . $keyIndex, cfg('INTERNAL_SIP_PORT'));
        $this->hexSetting('BS_IP_Data1.uiI_SIP_REGISTRAR_PORT' . $keyIndex, cfg('INTERNAL_SIP_PORT'));
        $this->stringSetting('BS_IP_Data1.aucS_SIP_REGISTRAR' . $keyIndex, $SipAccount->getSIPRegistrar());

        $this->setting('BS_IP_Data1.ucI_SIP_PREFERRED_TRANSPORT_LAYER' . $keyIndex, '6');
    }

    private function loadConfigValuesForN720()
    {
        $SipAccount = $this->sipAccount;
        //Provisioning-Settings
        $this->stringSetting('BS_VOIP_Data.astVoipProviders[0].aucSIP_PROVIDER_NAME', cfg('PHONALISA_NAME'));
        $this->stringSetting('BS_VOIP_Data.astVoipProviders[0].aucSIP_DOMAIN', $SipAccount->getSIPRegistrar());
        $this->stringSetting('BS_VOIP_Data.astVoipProviders[0].aucSIP_SERVER', $SipAccount->getSIPRegistrar());
        $this->hexSetting('BS_VOIP_Data.astVoipProviders[0].uiSIP_SERVER_PORT', cfg('INTERNAL_SIP_PORT'));
        $this->hexSetting('BS_VOIP_Data.astVoipProviders[0].uiSIP_REGISTRAR_PORT', cfg('INTERNAL_SIP_PORT'));
        $this->stringSetting('BS_VOIP_Data.astVoipProviders[0].aucSIP_REGISTRAR', $SipAccount->getSIPRegistrar());

        $this->setting('BS_XML_Netdirs.aucNetdirSelForDirectAccess[%]', '3');

        $this->setting('BS_IP_Data.ucB_USE_R_KEY_FOR_CALL_TRANSFER', '1');

        $this->configureSipAccountN720($SipAccount, 0);
        $this->hexSetting('BS_VOIP_Data.astVoipProviders[0].uiRE_REGISTRATION_TIMER', $SipAccount->expires);
        $this->hexSetting('BS_VOIP_Data.astVoipProviders[%].uiRE_REGISTRATION_TIMER', $SipAccount->expires);

        $Phone = $this->getPhoneModel();
        $this->removeOvermuchPhones($Phone);
        // enumerating devices in database
        for ($i = 1; $i < GigasetPhoneProvisioning::MAX_DEVICES_N720; $i++) {
            $phone = Phones::model()->findByAttributes(array("mac_addr" => $this->mMacAddress . "-" . $i));

            if (!$phone && $i < cfg('GIGASET_DECT_DEVICE_MAX')) {
                $phone = $this->getAdditionalPhone($i);
            }

            if (!$phone) {
                $this->setting('BS_VOIP_Data.astVoipAccounts[' . $i . '].ucB_SIP_ACCOUNT_IS_ACTIVE_1', '0');
                $this->setting('BS_VOIP_Data.astVoipAccounts[' . $i . '].ucB_VOIP_NET_AM_ENABLED_1', '0');
            } else {
                $SipAccount = $phone->getSipAccount();
                $this->configureSipAccountN720($SipAccount, $i);
            }
        }

        $this->setting('BS_IP_Data.ucI_HTTPLANGUAGE', $this->getLanguage());
        $this->setting('BS_VOIP_Data.ucB_USE_R_KEY_FOR_CALL_TRANSFER', $this->getLanguage());

        // XML-Phonebook
        $this->stringSetting('BS_XML_Netdirs.astNetdirProvider[1].aucServerURL[%]', $this->getPhonebookURL());
        $this->stringSetting('BS_XML_Netdirs.astNetdirProvider[1].aucWhitePagesDirName[%]', __('ADDRESSESS_FORM_HEADER_BUSINESS'));
        $this->stringSetting('BS_XML_Netdirs.astNetdirProvider[1].aucPrivatePagesDirName[%]', __('ADDRESSESS_FORM_HEADER_PRIVATE'));
        $this->setting('BS_XML_Netdirs.aucActivatedNetdirs[%]', $this->gigasetHexFormat(20) . ',' . $this->gigasetHexFormat(21));
        $this->setting('BS_XML_Netdirs.aucAvailableNetdirs[%]', $this->gigasetHexFormat(20) . ',' . $this->gigasetHexFormat(21));

        //NTP-Settings
        $this->setting('BS_IP_Data.ucB_TIME_USE_AUTOMATIC_NTP_SYN', '1');
        $this->stringSetting('BS_IP_Data.aucS_TIME_NTP_SERVER', cfg('NTP_HOST'));
        $this->setting('BS_IP_Data.ucB_TIME_USE_AUTOMATIC_DST', '1');
        $this->hexSetting('BS_IP_Data.uiI_TIME_TIMEZONE', $this->getTimeZone());

        //Audio-Codec-Settings
        $this->setting('BS_VOIP_Data.astVoipAccounts[%].aucI_SIP_PREFERRED_VOCODER[%]', $this->getPhoneCodecs());
        $this->setting('BS_VOIP_Data.ucI_SIP_AVAILABLE_VOCODER[%]', $this->getPhoneCodecs());
        $this->setting('BS_VOIP_Data.ucI_SIP_PREDEFINED_VOCODER[%]', $this->getPhoneCodecs());

        $this->setting('BS_VOIP_Data.astVoipProviders[%].ucSIP_PREFERRED_TRANSPORT_LAYER', '6');

        $this->stringSetting('BS_VOIP_Data.astVoipAccounts[%].aucS_VOIP_NET_AM_NUMBER_1', cfg('DP_VOICEMAIL_LISTENING_PREFIX'));
        $this->hexSetting('BS_IP_Data.ucI_DIALING_PLAN_COUNTRY_ID', $this->getDialingPlanCountryID());
    }

    private function configureSipAccountN720($SipAccount, $index)
    {
        $this->stringSetting('BS_VOIP_Data.astVoipAccounts[' . $index . '].aucS_SIP_DISPLAYNAME', $SipAccount->authname);
        $this->stringSetting('BS_VOIP_Data.astVoipAccounts[' . $index . '].aucS_SIP_LOGIN_ID', $SipAccount->authname);
        $this->stringSetting('BS_VOIP_Data.astVoipAccounts[' . $index . '].aucS_SIP_PASSWORD', $SipAccount->secret);
        $this->stringSetting('BS_VOIP_Data.astVoipAccounts[' . $index . '].aucS_SIP_USER_ID', $SipAccount->authname);
        $this->stringSetting('BS_VOIP_Data.astVoipAccounts[' . $index . '].aucS_SIP_ACCOUNT_NAME_1', $SipAccount->authname);
        $this->setting('BS_VOIP_Data.astVoipAccounts[' . $index . '].ucB_SIP_ACCOUNT_IS_ACTIVE_1', '1');
        $this->setting('BS_VOIP_Data.astVoipAccounts[' . $index . '].ucB_VOIP_NET_AM_ENABLED_1', '1');
    }

    private function loadConfigValuesForN870()
    {
        $this->n870Setting('DeleteNotProvisionedHS', '', 'oper'); // Delete/Remove not used Handsets
        $this->n870Setting('WebUI.1.Language', $this->getLanguageN870());

        // VoIP Provider
        $this->n870Setting('SipProvider.0.Name', cfg('PHONALISA_NAME'));

        // Phone system
        $this->n870Setting('SipProvider.0.PhoneSystem', 0);

        // Service provider
        $this->n870Setting('SipProvider.0.Domain', cfg('SIP_HOST'));
        $this->n870Setting('SipProvider.0.ProxyServerAddress', cfg('SIP_HOST'));
        $this->n870Setting('SipProvider.0.RegServerAddress', cfg('SIP_HOST'));
        $this->n870Setting('SipProvider.0.RegServerRefreshTimer', $this->getExpireTimeN870());

        if (cfg('INTERNAL_SSL_ENABLE')) {
            $this->n870Setting('SipProvider.0.RegServerPort', cfg('INTERNAL_TLS_PORT'));
            $this->n870Setting('SipProvider.0.TransportProtocol', 3); //1 = UDP; 2 = TCP; 3 = TLS
            $this->n870Setting('SipProvider.0.ProxyServerPort', cfg('INTERNAL_TLS_PORT'));
        } else {
            $this->n870Setting('SipProvider.0.RegServerPort', cfg('INTERNAL_SIP_PORT'));
            $this->n870Setting('SipProvider.0.TransportProtocol', 2); //1 = UDP; 2 = TCP; 3 = TLS
            $this->n870Setting('SipProvider.0.ProxyServerPort', cfg('INTERNAL_SIP_PORT'));
        }

        $this->n870Setting('SipProvider.0.UseSIPS', 0);
        if (cfg('RTP_ENABLE_SRTP')) {
            $this->n870Setting('SipProvider.0.SRTP_Enabled', 1);
        } else {
            $this->n870Setting('SipProvider.0.SRTP_Enabled', 0);
        }

        $this->n870Setting('SipProvider.0.AcceptNonSRTPCalls', 1);

        // Failover Server
        $this->n870Setting('SipProvider.0.FailoverServerEnabled', 0);

        /* DTMF over VoIP Connections
         * 1 = Audio;
         * 2 = RFC 2833;
         * 3 = Audio + RFC 2833;
         * 4 = SIP INFO;
         * 5 = Audio + SIP INFO;
         * 6 = RFC 2833 + SIP INFO;
         * 7 = Audio + RFC 2833 + SIP INFO;
         * 129 = Automatic DTMF Negotiation
         */
        $this->n870Setting('SipProvider.0.DTMFTransmission', 4);

        // Codecs
        $this->n870Setting('SipProvider.0.ActiveCodecs', $this->getPhoneCodecsN870());

        // Tone Scheme
        $this->n870Setting('Telephony.0.ToneScheme', $this->getToneSchemeN870());

        // SIP
        $this->n870Setting('DmGlobal.0.Port', cfg('INTERNAL_SIP_PORT'));
        $this->n870Setting('DmGlobal.0.TLS_Port', cfg('INTERNAL_TLS_PORT'));
        $this->n870Setting('DmGlobal.0.Timers_Session', $this->getExpireTimeN870(4));
        $this->n870Setting('DmGlobal.0.Timers_Subscription', $this->getExpireTimeN870(4));

        // Date and time
        $this->n870Setting('DmGlobal.0.NtpServer', cfg('NTP_HOST'));
        $this->n870Setting('DmGlobal.0.TimeZone', $this->getTimeZoneN870());

        // Provisioning-Settings
        $this->n870Setting('Provisioning.global.ProvisioningServer', $this->getProvURL());

        // Displayed name in idle display
        $this->n870Setting('DmGlobal.0.HSIdleDisplay', 1); //0 = Username; 1 = Display name

        $Phone = $this->getPhoneModel();
        $this->configureSipAccountN870($this->sipAccount, 1);
        $this->removeOvermuchPhones($Phone);

        // enumerating devices in database
        for ($i = 2; $i <= $this->getMaxPhones($Phone); $i++) {
            $phone = Phones::model()->findByAttributes(array("mac_addr" => $this->mMacAddress . "-" . ($i - 1)));

            if (!$phone) {
                $phone = $this->getAdditionalPhone($i - 1);
            }

            if ($phone) {
                $SipAccount = $phone->getSipAccount();
                $this->configureSipAccountN870($SipAccount, $i);
            }
        }

        $label = cfg('PHONALISA_NAME');
        if (strlen($label) >= 20) {
            $label = substr($label, 0, 15) . '...';
        }

        // Phonebook
        $this->n870Setting('XMLDir.0.ProviderName', $label);
        $this->n870Setting('XMLDir.0.ServerURL', $this->getPhonebookURL());

        // Public White Pages
        $this->n870Setting('Netdir.4.Activated', 1);
        $this->n870Setting('XMLDir.0.WhitePagesDirName', __('ADDRESSESS_FORM_HEADER_BUSINESS'));
        $this->n870Setting('XMLDir.0.StartWithListWP', 1);

        // Public Private Pages
        $this->n870Setting('Netdir.6.Activated', 1);
        $this->n870Setting('XMLDir.0.PrivatePagesDirName', __('ADDRESSESS_FORM_HEADER_PRIVATE'));
        $this->n870Setting('XMLDir.0.StartWithListPP', 1);

        // SIP-ID for Phonebooks
        $this->n870Setting('DmGlobal.0.AddSipId', 1);

        $this->n870Setting('Netdir.5.Activated', 0);
    }

    protected function removeOvermuchPhones($Phone)
    {
        $phones = Phones::model()->find(array("mac_addr" => $this->mMacAddress . "*"), 'mac_addr DESC');
        $maxPhones = $this->getMaxPhones($Phone);
        $count = count($phones);
        $diff = $maxPhones - $count;
        usort($phones, array($this, "cmp"));

        if ($diff < 0) {
            $phonesToDelete = array_slice($phones, $diff);
            foreach ($phonesToDelete as $phoneToDelete) {
                $phoneToDelete->delete();
            }
        }
    }

    protected function cmp($a, $b)
    {
        $sortedValues = array(
            $a->mac_addr,
            $b->mac_addr,
        );

        natsort($sortedValues);
        if (array_values($sortedValues)[0] == $a->mac_addr) {
            return -1;
        } else {
            return 1;
        }
    }

    protected function configureSipAccountN870($SipAccount, $id)
    {
        $ipui = str_pad($id, 5, '0', STR_PAD_LEFT);
        $this->n870Setting("add_hs.<$ipui>", $ipui, '<oper>');
        $this->n870Setting("hs.RegStatus.<$ipui>", 'ToReg', 'param</oper>');
        $this->n870Setting("hs.$ipui.DECT_AC", '0000');
        $this->n870Setting("SipAccount.$ipui.Active", '1');
        $this->n870Setting("SipAccount.$ipui.AuthName", $SipAccount->authname);
        $this->n870Setting("SipAccount.$ipui.AuthPassword", $SipAccount->secret);
        $this->n870Setting("SipAccount.$ipui.ProviderId", '0');
        $this->n870Setting("SipAccount.$ipui.UserName", $SipAccount->authname);
        $this->n870Setting("SipAccount.$ipui.DisplayName", $SipAccount->authname);
        $this->n870Setting("SipAccount.$ipui.VoiceMailMailbox", cfg('DP_VOICEMAIL_LISTENING_PREFIX'));
    }

    protected function getExpireTimeN870($maxDigits = 5)
    {
        $SipAccount = $this->sipAccount;
        $expireTime = $SipAccount->expires;
        if (strlen($expireTime) > $maxDigits) {
            $expireTime = str_pad(9, $maxDigits, 9, STR_PAD_LEFT);
        }
        return $expireTime;
    }

    private function getTemplate()
    {
        // Read phone specific template if this exists
        $filename = dirname(__FILE__) . "/templates/" . $this->mCurrentPhoneModel->getPhoneModelName() . ".xml";
        if (file_exists($filename)) {
            $this->readTemplateFile($filename);
        }
        return $this->mCurrentSettings;
    }

    private function readTemplateFile($filename)
    {
        $data = file_get_contents($filename);
        $xml = new SimpleXMLElement($data);
        foreach ($xml as $section => $element) {
            foreach ($element as $key => $value) {
                if (strtoupper($key) != 'SYMB_ITEM') {
                    continue;
                }
                $this->setting((string) $value["ID"], (string) $value["value"], (string) $value["class"]);
            }
        }
    }

    protected function stringSetting($key, $value = '')
    {
        $this->setting($key, '"' . $value . '"');
    }

    protected function hexSetting($key, $value = '')
    {
        $this->setting($key, $this->gigasetHexFormat($value));
    }

    protected function gigasetHexFormat($value)
    {
        return '0x' . dechex($value);
    }

    protected function setting($key, $value = '', $class = 'symb_item')
    {
        $setting = array();
        $setting[$key]["class"] = $class;
        $setting[$key]["value"] = $value;
        $this->mCurrentSettings[$key] = $setting[$key];
    }

    protected function n870Setting($key, $value = '', $class = 'param')
    {
        $this->setting($key, $value, $class);
    }

    protected function generateOutput()
    {
        $xw = new xmlWriter();
        $xw->openMemory();
        $xw->setIndent(TRUE);
        $xw->startDocument('1.0', 'UTF-8'); // start document

        if (startsWith($this->getPhoneModel()->type, 'N870')) {
            $xw = $this->getXmlSchemaForN870($xw);
        } else {
            $xw = $this->getXmlSchema($xw);
        }

        $xw->endDocument(); // end document

        $xml = $xw->outputMemory();
        echo str_replace(array('"&quot;', '&quot;"'), array("'\"", "\"'"), $xml);
        return true;
    }

    protected function getXmlSchemaForN870($xw)
    {
        $xw->startElement('provisioning');
        $xw->writeAttribute('version', '1.1');
        $xw->writeAttribute('productID', 'e2');

        $xw->startElement('nvm');
        $ipui = '';
        foreach ($this->mCurrentSettings as $section_name => $section_values) {
            if (startsWith($section_values['class'], '<oper>')) {
                $ipui = $section_values['value'];
                $section_values['class'] = str_replace(array('<', '>'), '', $section_values['class']);
                $section_name = str_replace(".<$ipui>", '', $section_name);
                $this->writeSectionElements($xw, $section_values, $section_name);
            } else if (endsWith($section_values['class'], '</oper>')) {
                $section_values['class'] = str_replace('</oper>', '', $section_values['class']);
                $section_name = str_replace(".<$ipui>", '', $section_name);
                $this->writeSectionElements($xw, $section_values, $section_name);
                $ipui = '';
                $xw->endElement();
                $xw->endElement(); //</oper>
            } else {
                $this->writeSectionElements($xw, $section_values, $section_name);
                $xw->endElement();
            }
        }
        $xw->endElement();  //</nvm>

        $xw->endElement(); //</provisioning>
        return $xw;
    }

    protected function writeSectionElements($xw, $section_values, $section_name)
    {
        $xw->startElement($section_values['class']);
        $xw->writeAttribute('name', $section_name);
        $xw->writeAttribute('value', $section_values['value']);
    }

    protected function getXMLSchema($xw)
    {
        $xw->startElement('ProviderFrame');
        $xw->startElement('Provider');

        $xw->startElement('MAC_ADDRESS');
        $xw->writeAttribute('value', @$this->getPhoneModel()->mac_addr);
        $xw->endElement();

        # This is removed to test, if the provisioning not longer need a minute to starts
        //$xw->startElement('VERSION');
        //$xw->writeAttribute('value', PhonalisaDate::convertServerDateTo('dmyHi', PhonalisaDate::newServerDate()));
        //$xw->endElement();

        $xw->startElement('PROFILE_NAME');
        $xw->writeAttribute('class', 'string');
        $xw->writeAttribute('value', @$this->getPhoneModel()->type);
        $xw->endElement();

        foreach ($this->mCurrentSettings as $section_name => $section_values) {
            $xw->startElement('SYMB_ITEM');
            $xw->writeAttribute('ID', $section_name);
            $xw->writeAttribute('class', $section_values['class']);
            $xw->writeAttribute('value', $section_values['value']);
            $xw->endElement();
        }

        $xw->endElement();
        $xw->endElement();

        return $xw;
    }

    protected function generateNewPhone()
    {
        $phone = $this->generatePhoneForGigasetWithMac($this->mMacAddress);
        for ($i = 1; $i < $this->getMaxPhones($phone); $i++) {
            $this->generatePhoneForGigasetWithMac($this->mMacAddress . "-" . $i);
        }
    }

    protected function getAdditionalPhone($macID)
    {
        $this->generatePhoneForGigasetWithMac($this->mMacAddress . "-" . $macID);
        return Phones::model()->findByAttributes(array("mac_addr" => $this->mMacAddress . "-" . $macID));
    }

    private function getMaxPhones($phone)
    {
        $sysConfigPhonesMax = cfg('GIGASET_DECT_DEVICE_MAX');
        if (!isset($sysConfigPhonesMax)) {
            $sysConfigPhonesMax = 10;
        }

        $maxPhones = (int) $sysConfigPhonesMax;
        if (startsWith($phone->type, 'N510') && $maxPhones > 6) {
            $maxPhones = 6;
        }
        return $maxPhones;
    }

    private function generatePhoneForGigasetWithMac($mac)
    {
        $model = new Phones();
        $model->mac_addr = $mac;
        $model->type = $this->mCurrentPhoneModel->getPhoneModelName();
        $model->nobody_index = 1;
        $model->added = 1;
        $model->firmware = $this->mCurrentFirmwareVersion;
        $model->fw_manual_update = 0;
        $model->ipv4_address = $this->mIPv4Address;
        $model->setIsNewRecord(true);

        //generate dummy sip account for this phone
        $sipaccount = new SipAccounts();
        $sipaccount->authname = $this->getNextFreeDummyNumber();
        $sipaccount->secret = PasswordGenerator::generateSipPassword();
        $sipaccount->setIsNewRecord(true);
        $sipaccount->save(false);

        $model->sip_account_id = $sipaccount->id;
        $model->nobody_index = $sipaccount->id;

        if (!$model->save(false)) {
            foreach ($model->errors as $errors) {
                foreach ($errors as $key => $value) {
                    Log::d("$key => $value", "gigaset-dls");
                }
            }
        }
        return $model;
    }

    protected function getLanguage()
    {
        $langCodeList = array(
            'en_us' => '0x01',
            'de_de' => '0x02',
            'fr_fr' => '0x09',
            'nl_nl' => '0x0a'
        );

        $defaultLangCode = '0x01';
        return $this->getLanguageFromLangCodeList($langCodeList, $defaultLangCode);
    }

    protected function getLanguageN870()
    {
        $langCodeList = array(
            'en_us' => 'en-bt',
            'de_de' => 'de-de',
            'fr_fr' => 'fr-fr',
            'nl_nl' => 'nl-nl'
        );

        $defaultLangCode = 'en-bt';
        return $this->getLanguageFromLangCodeList($langCodeList, $defaultLangCode);
    }

    protected function getLanguageFromLangCodeList($langCodeList, $defaultLangCode)
    {
        $guiLanguage = cfg('GUI_LANGS');
        $language = $defaultLangCode;
        if (isset($langCodeList[$guiLanguage])) {
            $language = $langCodeList[$guiLanguage];
        }
        return $language;
    }

    protected function getTimeZone()
    {
        $timeZone = new DateTimeZone(cfg('TIMEZONE'));
        $dateTime = new DateTime("now", $timeZone);
        $gmtDifference = $dateTime->format('P');
        if ($dateTime->format('I')) {
            $exploded = explode(':', $gmtDifference);
            if (isset($exploded[0]) && isset($exploded[1])) {
                $sign = substr($gmtDifference, 0, 1);
                $hourWithoutDST = ((int) ltrim($exploded[0], $sign) - 1);
                $gmtDifference = $sign . str_pad($hourWithoutDST, 2, '0', STR_PAD_LEFT) . ':' . $exploded[1];
            }
        }

        $timezoneCodes = $this->getGigasetTimeZoneCode();

        $timezone = '26';
        if (isset($timezoneCodes[$gmtDifference])) {
            $timezone = $timezoneCodes[$gmtDifference];
        }
        return $timezone;
    }

    protected function getGigasetTimeZoneCode()
    {
        return array(
            '-12:00' => '0', //(GMT-12:00) International Date Line West
            '-11:00' => '1', //(GMT-11:00) Midway Island, Samoa
            '-10:00' => '2', //(GMT-10:00) Hawaii
            '-09:00' => '3', //(GMT-09:00) Alaska
            '-08:00' => '4', //(GMT-08:00) Pacific Time (US & Canada), Tijuana
            '-07:00' => '7', //(GMT-07:00) Mountain Time (US & Canada)
            '-06:00' => '9', //(GMT-06:00) Central Time (US & Canada)
            '-05:00' => '13', //(GMT-05:00) Eastern Time (US & Canada)
            '-04:00' => '15', //(GMT-04:00) Atlantic Time (Canada)
            '-03:30' => '18', //(GMT-03:30) Newfoundland
            '-03:00' => '21', //(GMT-03:00) Greenland
            '-02:00' => '22', //(GMT-02:00) Mid-Atlantic
            '-01:00' => '23', //(GMT-01:00) Azores
            '+00:00' => '26', //(GMT±00:00) Greenwich Mean Time : Dublin, Edinburgh, Lisbon, London
            '+01:00' => '27', //(GMT+01:00) Amsterdam, Berlin, Bern, Rome, Stockholm, Vienna
            '+02:00' => '36', //(GMT+02:00) Helsinki, Kyiv, Riga, Sofia, Tallinn, Vilnius
            '+03:00' => '38', //(GMT+03:00) Baghdad
            '+03:30' => '42', //(GMT+03:30) Tehran
            '+04:00' => '43', //(GMT+04:00) Abu Dhabi, Muscat
            '+04:30' => '45', //(GMT+04:30) Kabul
            '+05:00' => '47', //(GMT+05:00) Islamabad, Karachi, Tashkent
            '+05:30' => '48', //(GMT+05:30) Chennai, Kolkata, Mumbai, New Delhi
            '+05:45' => '49', //(GMT+05:45) Kathmandu
            '+06:00' => '50', //(GMT+06:00) Almaty, Novosibirsk
            '+06:30' => '53', //(GMT+06:30) Rangoon
            '+07:00' => '54', //(GMT+07:00) Bangkok, Hanoi, Jakarta
            '+08:00' => '56', //(GMT+08:00) Beijing, Chongqing, Hong Kong, Urumqi
            '+09:00' => '62', //(GMT+09:00) Seoul
            '+09:30' => '65', //(GMT+09:30) Darwin
            '+10:00' => '67', //(GMT+10:00) Canberra, Melbourne, Sydney
            '+11:00' => '71', //(GMT+11:00) Magadan, Solomon Is., New Caledonia
            '+12:00' => '72', //(GMT+12:00) Auckland, Wellington
            '+13:00' => '74' //(GMT+13:00) Nuku'alofa
        );
    }

    protected function getTimeZoneN870()
    {
        $timeZone = cfg('TIMEZONE');
        return str_replace('_', ' ', $timeZone);
    }

    protected function getToneScheme()
    {
        $codeList = array(
            'en_us' => '1',
            'de_de' => '12',
            'fr_fr' => '7',
            'nl_nl' => '9'
        );
        $defaultCode = '1';
        return $this->getToneSchemeByCodeList($codeList, $defaultCode);
    }

    protected function getToneSchemeN870()
    {
        $codeList = array(
            'en_us' => 'UK',
            'de_de' => 'Germany',
            'fr_fr' => 'France',
            'nl_nl' => 'Netherlands'
        );

        $defaultCode = 'UK';
        return $this->getToneSchemeByCodeList($codeList, $defaultCode);
    }

    protected function getToneSchemeByCodeList($codeList, $defaultCode)
    {
        $guiLanguage = cfg('GUI_LANGS');

        $toneScheme = $defaultCode;
        if (isset($codeList[$guiLanguage])) {
            $toneScheme = $codeList[$guiLanguage];
        }
        return $toneScheme;
    }

    protected function getPhoneCodecs()
    {
        $codecList = array(
            'PCMU' => '0x00',
            'PCMA' => '0x01',
            'G726' => '0x02',
            'G729' => '0x03',
            'G726' => '0x04',
            'G722' => '0x05'
        );
        return $this->getPhoneCodecsByList($codecList);
    }

    protected function getPhoneCodecsN870()
    {
        $codecList = array(
            'PCMU' => 0,
            'PCMA' => 8,
            'G722' => 9,
            'G729' => 18
        );
        return $this->getPhoneCodecsByList($codecList);
    }

    protected function getPhoneCodecsByList($codecList)
    {
        $phonalisaCodecs = str_replace(' ', '', cfg('GLOBAL_CODEC_PREFS'));
        $codecs = explode(',', $phonalisaCodecs);

        $phoneCodec = '';
        foreach ($codecs as $codec) {
            if (isset($codecList[$codec])) {
                $phoneCodec .= $codecList[$codec] . ',';
            }
        }
        return trim($phoneCodec, ',');
    }

    protected function getDialingPlanCountryID()
    {
        $guiLanguage = cfg('GUI_LANGS');
        $countryCodes = array(
            'en_us' => '75',
            'de_de' => '25',
            'fr_fr' => '24',
            'nl_nl' => '48'
        );

        $dialingCountryID = '25';
        if (isset($countryCodes[$guiLanguage])) {
            $dialingCountryID = $countryCodes[$guiLanguage];
        }
        return $dialingCountryID;
    }
}
